package org.jboss.seam.wiki.core.action;

import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.annotations.AutoCreate;
import org.jboss.seam.annotations.web.RequestParameter;
import org.jboss.seam.ScopeType;

import java.io.Serializable;

@Name("pager")
@Scope(ScopeType.PAGE)
@AutoCreate
public class Pager implements Serializable {

    private Long numOfRecords = 0l;
    private Integer page = 0;
    private Long pageSize = 15l;

    public Pager() {}

    public Pager(Long pageSize) {
        this.pageSize = pageSize;
    }

    public Long getNumOfRecords() {
        return numOfRecords;
    }

    public void setNumOfRecords(Long numOfRecords) {
        this.numOfRecords = numOfRecords;
    }

    public Integer getPage() {
        return page;
    }

    public void setPage(Integer page) {
        if (page != null) this.page = page;
    }

    public Long getPageSize() {
        return pageSize;
    }

    @RequestParameter
    public void setPageSize(Long pageSize) {
        if (pageSize != null)
            this.pageSize = pageSize;
    }

    public int getNextPage() {
        return page + 1;
    }

    public int getPreviousPage() {
        return page - 1;
    }

    public int getFirstPage() {
        return 0;
    }

    public long getFirstRecord() {
        return page * pageSize + 1;
    }

    public long getLastRecord() {
        return (page * pageSize + pageSize) > numOfRecords
                ? numOfRecords
                : page * pageSize + pageSize;
    }

    public long getNextRecord() {
        return page * pageSize;
    }

    public long getLastPage() {
        long lastPage = (numOfRecords / pageSize);
        if (numOfRecords % pageSize == 0) lastPage--;
        return lastPage;
    }

    public boolean isNextPageAvailable() {
        return numOfRecords > ((page * pageSize) + pageSize);
    }

    public boolean isPreviousPageAvailable() {
        return numOfRecords != null && page > 0;
    }

}
